@@ -54,6 +54,7 @@
 #include <kio/netaccess.h>
 #include <KUrlMimeData>
 #include <KUrl>
+#include <KStringHandler>
 
 #include "menufile.h"
 #include "menuinfo.h"
@@ -140,7 +141,7 @@ bool TreeItem::itemNameLessThan(QTreeWidgetItem *item1, QTreeWidgetItem *item2)
 {
     TreeItem *treeItem1 = static_cast<TreeItem *>(item1);
     TreeItem *treeItem2 = static_cast<TreeItem *>(item2);
-    return treeItem1->name().toLower() < treeItem2->name().toLower();
+    return QString::localeAwareCompare(treeItem1->name(), treeItem2->name()) < 0;
 }
 
 /**
@@ -154,14 +155,14 @@ bool TreeItem::itemDescriptionLessThan(QTreeWidgetItem *item1, QTreeWidgetItem *
     // extract descriptions in lower case
     TreeItem *treeItem1 = static_cast<TreeItem *>(item1);
     TreeItem *treeItem2 = static_cast<TreeItem *>(item2);
-    const QString description1 = treeItem1->description().toLower();
-    const QString description2 = treeItem2->description().toLower();
+    const QString description1 = treeItem1->description();
+    const QString description2 = treeItem2->description();
 
     // if description is missing for both items, sort them using their names
     if (description1.isEmpty() && description2.isEmpty()) {
         return itemNameLessThan(item1, item2);
     } else {
-        return description1 < description2;
+        return QString::localeAwareCompare(description1, description2) < 0;
     }
 }
 
@@ -497,15 +498,6 @@ void TreeView::fillBranch(MenuFolderInfo *folderInfo, TreeItem *parent)
         }
     }
 }
-
-void TreeView::closeAllItems(QTreeWidgetItem *item)
-{
-    item->setExpanded(false);
-    for (int i = 0; i < item->childCount(); ++i) {
-        closeAllItems(item->child(i));
-    }
-}
-
 TreeItem *TreeView::expandPath(TreeItem *item, const QString &path)
 {
     int i = path.indexOf(QStringLiteral("/"));
@@ -534,9 +526,10 @@ TreeItem *TreeView::expandPath(TreeItem *item, const QString &path)
 
 void TreeView::selectMenu(const QString &menu)
 {
-    for (int i = 0; i < topLevelItemCount(); ++i) {
-        closeAllItems(topLevelItem(i));
-    }
+
+    // close all parent expansions and deselect everything
+    collapseAll();
+    setCurrentIndex(rootIndex());
 
     if (menu.length() <= 1) {
         setCurrentItem(topLevelItem(0));
@@ -1543,11 +1536,11 @@ void TreeView::sortItemChildren(const QList<QTreeWidgetItem *>::iterator &begin,
 {
     // sort by name
     if (sortType == SortByName) {
-        qSort(begin, end, TreeItem::itemNameLessThan);
+        std::sort(begin, end, TreeItem::itemNameLessThan);
     }
     // sort by description
     else if (sortType == SortByDescription) {
-        qSort(begin, end, TreeItem::itemDescriptionLessThan);
+        std::sort(begin, end, TreeItem::itemDescriptionLessThan);
     }
 }
 
@@ -1926,6 +1919,18 @@ void TreeView::sendReloadMenu()
     QDBusConnection::sessionBus().send(message);
 }
 
+// Slot to expand or retract items in tree based on length of search string
+void TreeView::searchUpdated(const QString &searchString) {
+    // expand all categories if we typed more than a few characters, otherwise collapse and un-select everything
+    // use logicalLength for CJK users
+    if (KStringHandler::logicalLength(searchString) > 2) {
+        expandAll();
+    } else {
+        collapseAll();
+        setCurrentIndex(rootIndex());
+    }
+}
+
 MenuItemMimeData::MenuItemMimeData(TreeItem *item)
     : QMimeData()
     , m_item(item)
